library zoomer;

import 'dart:math';
import 'package:flutter/material.dart';

class Zoomer extends StatefulWidget {
  ///Zoomer widget to create interactive Zoomable widget
  final Widget child;
  final double height,width,minScale,maxScale;
  final BoxDecoration background;
  final bool enableRotation;
  final bool clipRotation;
  final bool enableTranslation;
  final Function(ZoomerState) getInstance;
  Zoomer({this.child,this.height,this.width,this.background,this.maxScale = 2.0,this.minScale = 0.5,this.enableTranslation = false,this.enableRotation = false,this.clipRotation = true,this.getInstance});
  @override
  ZoomerState createState() => ZoomerState();
}

class ZoomerState extends State<Zoomer> {
  double _scale = -1.0;
  Offset _offset = Offset(0.0, 0.0);
  double _startScale = 1.0;
  Offset _lastOffset;
  Offset _startOffset;
  Offset _limitOffset;
  double _angle = 0.0;
  double _startAngle = 0.0;

  double get getScale=>-_scale;
  set setScale(double value)=> setState((){_scale = -value;});
  Offset get getOffset=>_offset;
  set setOffset(Offset value)=>setState((){_offset = value;});
  double get getRotation=>_angle;
  set setRotation(double value)=>setState((){_angle = value;});

  @override
  void initState(){
    super.initState();
    double l = (1+_scale);
    l = l<0?(-_scale-1)/4:l;
    _limitOffset = Offset(widget.width, widget.height)*l;

    if(widget.getInstance != null){
      widget.getInstance(this);
    }
  }

  void _scaleStart(ScaleStartDetails details){
    _lastOffset = details.focalPoint;
    _startScale = -_scale;
    _startOffset = _offset;
    _startAngle = _angle;
  }

  void _scaleEnd(ScaleEndDetails details){

    if(widget.enableTranslation){
      double l = (1+_scale);
      l = l<0?(-_scale-1)/4:l;
      _limitOffset = Offset(widget.width, widget.height)*l;
    }

    if(widget.enableRotation){
      if(widget.clipRotation){
        double x = ((_angle % (2*pi))/(pi/2));
        double angle = ((x-x.floor())>0.5?(x.floor()+1):x.floor()) * (pi/2);
        setState(() {
          _angle= angle;
        });
      }
    }
  }

  void _scaleUpdate(ScaleUpdateDetails details){
    double scale = (_startScale * details.scale).clamp(widget.minScale,widget.maxScale);
    Offset offset = details.focalPoint - ((_lastOffset - _startOffset)/_startScale) * scale;
    double angle = _startAngle+details.rotation;
    setState(() {
      _scale = -scale;

      if(widget.enableRotation){
        _angle = angle;
      }

      if(details.scale == 1.0 && widget.enableTranslation){
        _offset = Offset(offset.dx.clamp(-_limitOffset.dx, _limitOffset.dx),offset.dy.clamp(-_limitOffset.dy, _limitOffset.dy));
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onScaleStart: _scaleStart,
      onScaleUpdate: _scaleUpdate,
      onScaleEnd: _scaleEnd,
      child: ClipRect(
        child: Container(
          decoration: widget.background==null?BoxDecoration(color: Colors.black):widget.background,
          height: widget.height,
          width: widget.width,
          child: Transform(transform:Matrix4.identity()..scale(-_scale,-_scale)..translate(_offset.dx,_offset.dy)..rotateZ(_angle),
            alignment: FractionalOffset.center,
            child:widget.child,
          ),
        ),
      ),
    );
  }
}